%{ open Ast %}

%token SEMI LPAREN RPAREN
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token SQUOTE DQUOTE
%token IF FOR WHILE PRINT INPUT
%token <bool> BOOLEAN
%token <int> INTEGER
%token <string> ID
%token <string> FNAME
%token EOF

%start program
%type <Ast.program> program

%%
program:
   /* nothing */ { [], [] }
 | program vdecl { ($2 :: fst $1), snd $1 }
 | program fdecl { fst $1, ($2 :: snd $1) }
	
expr:
	LITERAL				{ Literal($1) }
|	ID					{ Id($1) }
|	LPAREN op RPAREN	{ $2 }
|	LPAREN funcs RPAREN	{ $2 }
|						{ Noexpr }

expr_list:
	expr expr_list		{ $1 $2 }
|	expr				{ $1 }

op:
	arith_op			{ $1 }
|	assign				{ $1 }
|	cond_op				{ $1 }

arith_op:
|	PLUS expr expr		{ Oper(Add, $2, $3) }
|	MINUS expr expr		{ Oper(Sub, $2, $3) }
|	TIMES expr expr		{ Oper(Mult, $2, $3) }
|	DIVIDE expr expr	{ Oper(Div, $2, $3) }

assign:
|	ASSIGN expr expr	{ Oper(Assign, $2, $3) }

cond_op:
|	EQ expr expr		{ Oper(Equal, $2, $3) }
|	NEQ expr expr		{ Oper(Neq, $2, $3) }
|	LT expr expr		{ Oper(Less, $2, $3) }
|	LEQ expr expr		{ Oper(Leq, $2, $3) }
|	GT expr expr		{ Oper(Greater, $2, $3) }
|	GEQ expr expr		{ Oper(Geq, $2, $3) }	

funcs:
	IF cond_op expr		{ Call(If, $2, $3) }
	WHILE cond_op expr	{ Call(While, $2, $3) }
	FOR cond_op expr	{ Call(For, $2, $3) }
	PRINT cond_op expr	{ Call(Print, $2, $3) }
	INPUT cond_op expr	{ Call(Input, $2, $3) }
	CALL expr_list		{ Call(Fname, $1) }


